@@ -1,7 +1,7 @@
 /*
  *   Jackbeat - JACK sequencer
  *    
- *   Copyright (c) 2004-2006 Olivier Guilyardi <olivier {at} samalyse {dot} com>
+ *   Copyright (c) 2004-2008 Olivier Guilyardi <olivier {at} samalyse {dot} com>
  *    
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   SVN:$Id: error.c 39 2006-11-28 14:37:30Z olivier $
+ *   SVN:$Id: error.c 121 2008-01-10 00:17:40Z olivier $
  */
 #include <string.h>
 
@@ -29,8 +29,8 @@ char * error_to_string (int code)
   char *error_string;
   switch (code)
    {
-    case ERR_SEQUENCE_MSQ :
-     error_string = strdup ("Unable to allocate a new message queue. You may need" 
+    case ERR_SEQUENCE_MSQ : // obsolete
+     error_string = strdup ("Unable to allocate a new message queue. You may need " 
                             "to increase the MSGMNI Linux kernel parameter.");
      break;
     case ERR_SEQUENCE_JACK_CONNECT :
@@ -39,6 +39,10 @@ char * error_to_string (int code)
     case ERR_SEQUENCE_JACK_ACTIVATE :
      error_string = strdup ("Unable to activate the JACK client.");
      break;
+    case ERR_SEQUENCE_JACK_REGPORT :
+     error_string = strdup ("Unable to register a new JACK Port. You might need "
+                            "to increase the JACK server maximum number of ports.");
+     break;
     case ERR_JAB_XML :
      error_string = strdup ("The XML definition of your .jab file seems "
                             "corrupted. The operation can't complete.");
