@@ -1,7 +1,7 @@
 /*
  *   Jackbeat - JACK sequencer
  *    
- *   Copyright (c) 2004-2006 Olivier Guilyardi <olivier {at} samalyse {dot} com>
+ *   Copyright (c) 2004-2008 Olivier Guilyardi <olivier {at} samalyse {dot} com>
  *    
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,18 +17,19 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   SVN:$Id: error.h 39 2006-11-28 14:37:30Z olivier $
+ *   SVN:$Id: error.h 121 2008-01-10 00:17:40Z olivier $
  */
 
 #ifndef JACKBEAT_ERR_H
 #define JACKBEAT_ERR_H
 
 
-#define ERR_JAB_XML 1
-#define ERR_SEQUENCE_INTERNAL 2
-#define ERR_SEQUENCE_MSQ 3
-#define ERR_SEQUENCE_JACK_CONNECT 4
-#define ERR_SEQUENCE_JACK_ACTIVATE 5
+#define ERR_JAB_XML                 1
+#define ERR_SEQUENCE_INTERNAL       2
+#define ERR_SEQUENCE_MSQ            3 // obsolete
+#define ERR_SEQUENCE_JACK_CONNECT   4
+#define ERR_SEQUENCE_JACK_ACTIVATE  5
+#define ERR_SEQUENCE_JACK_REGPORT   6
 
 char * error_to_string (int code);
 
