@@ -1,7 +1,7 @@
 /*
  *   Jackbeat - JACK sequencer
  *    
- *   Copyright (c) 2004-2006 Olivier Guilyardi <olivier {at} samalyse {dot} com>
+ *   Copyright (c) 2004-2008 Olivier Guilyardi <olivier {at} samalyse {dot} com>
  *    
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,12 +17,13 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   SVN:$Id: grid.c 68 2007-03-01 12:02:41Z olivier $
+ *   SVN:$Id: grid.c 121 2008-01-10 00:17:40Z olivier $
  */
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <gdk/gdkkeysyms.h>
 #include "grid.h"
 
 #define GRID_ANIM_INTERVAL 30 
@@ -130,6 +131,7 @@ struct grid_t
   int       last_mask;
   int       pointed_col;
   int       pointed_row;
+  int       mask_pointer;
   int       animation_tag;
 
   grid_gc_t *gc;
@@ -138,13 +140,14 @@ struct grid_t
 
 static gboolean grid_configure_event (GtkWidget *widget, GdkEventConfigure *event, grid_t *grid);
 static gboolean grid_expose_event (GtkWidget *widget, GdkEventExpose *event, grid_t *grid);
-static void grid_size_request_event (GtkWidget *widget, GtkRequisition *requisition, grid_t *grid);
+static void     grid_size_request_event (GtkWidget *widget, GtkRequisition *requisition, grid_t *grid);
 static gboolean grid_button_press_event (GtkWidget *widget, GdkEventButton *event, grid_t *grid);
 static gboolean grid_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, grid_t *grid);
-static void grid_area_create (grid_t *grid);
-gboolean grid_animation_cb (gpointer data);
-static void grid_draw_cell (grid_t *grid, int col, int row, int Xnotify, int direct);
-static void grid_draw_all (grid_t *grid);
+static gboolean grid_key_action_event (GtkWidget * widget, GdkEventKey *event, grid_t * grid);
+static void     grid_area_create (grid_t *grid);
+gboolean        grid_animation_cb (gpointer data);
+static void     grid_draw_cell (grid_t *grid, int col, int row, int Xnotify, int direct);
+static void     grid_draw_all (grid_t *grid);
 
 grid_t *grid_new ()
 {
@@ -183,8 +186,8 @@ grid_resize (grid_t *grid, int col_num, int row_num)
 {
   int row;
 
-  if (grid->area != NULL) printf("grid_resize() grid->area is set\n");
-  else printf("grid_resize() grid->area is NOT set\n");
+  if (grid->area != NULL) { DEBUG("grid->area is set"); }
+  else { DEBUG("grid->area is NOT set"); }
   
   //printf ("Setting array size to %d rows\n", row_num);
   grid->cells = realloc (grid->cells, row_num * sizeof (grid_cell_t*));
@@ -270,7 +273,7 @@ grid_highlight_cell (grid_t *grid, int col, int row, float level)
 {
   if ((level > 1) || (level < 0)) 
   {
-    printf("WARNING - BAD LEVEL FROM OUTSIDE : %f\n", level);
+    DEBUG("Warning - Bad level: %f", level);
   }
   level = level * GRID_ANIM_LEVEL_MAGNIFY;
   if (level > 1) level = 1;
@@ -379,9 +382,15 @@ grid_area_create (grid_t *grid)
 		      G_CALLBACK (grid_button_press_event), grid);
   g_signal_connect (G_OBJECT (grid->area),"motion-notify-event",
 		      G_CALLBACK (grid_motion_notify_event), grid);
+  GTK_WIDGET_SET_FLAGS (grid->area, GTK_CAN_FOCUS);          
+  g_signal_connect (G_OBJECT (grid->area), "key_press_event",
+                    G_CALLBACK (grid_key_action_event), grid);
+  g_signal_connect (G_OBJECT (grid->area), "key_release_event",
+                    G_CALLBACK (grid_key_action_event), grid);
 
   gtk_widget_set_events (grid->area, GDK_EXPOSURE_MASK
 			 // | GDK_LEAVE_NOTIFY_MASK
+			 | GDK_KEY_PRESS_MASK
 			 | GDK_BUTTON_PRESS_MASK
 			 | GDK_POINTER_MOTION_MASK
 			 | GDK_POINTER_MOTION_HINT_MASK);
@@ -730,6 +739,8 @@ grid_configure_event (GtkWidget *widget, GdkEventConfigure *event, grid_t *grid)
 static gboolean
 grid_expose_event (GtkWidget *widget, GdkEventExpose *event, grid_t *grid)
 {
+  // FIXME: need to indicate focus
+  // see: http://developer.gnome.org/doc/API/2.0/gtk/GtkDrawingArea.html
   gdk_draw_drawable(widget->window,
 		    widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
 		    grid->pixmap,
@@ -748,6 +759,42 @@ grid_size_request_event (GtkWidget *widget, GtkRequisition *requisition, grid_t
   DEBUG("size request called - width/height: %d/%d", requisition->width, requisition->height);
 }
 
+static void
+grid_toggle_cell (grid_t *grid, int col, int row, int mask)
+{
+  if (mask)  
+   {
+    grid->last_mask = grid->cells[row][col].mask = grid->cells[row][col].mask ? 0 : 1;
+    grid_draw_cell (grid, col, row, 1, 0);
+    grid->last_col = col;
+    grid->last_row = row;
+    if (grid->mask_changed_cb)
+      grid->mask_changed_cb (grid->mask_changed_data, col, row, grid->cells[row][col].mask);
+   }
+  else 
+   {
+    grid->last_value = grid->cells[row][col].value = grid->cells[row][col].value ? 0 : 1;
+    grid_draw_cell (grid, col, row, 1, 0);
+    grid->last_col = col;
+    grid->last_row = row;
+    if (grid->value_changed_cb)
+      grid->value_changed_cb (grid->value_changed_data, col, row, grid->cells[row][col].value);
+   }
+}
+
+static void
+grid_move_pointer (grid_t *grid, int col, int row, int mask)
+{
+  if (grid->pointed_col != -1 && grid->pointed_row != -1 &&
+      (grid->pointed_col != col || grid->pointed_row != row || grid->mask_pointer != mask))
+    grid_draw_cell (grid, grid->pointed_col, grid->pointed_row, 1, 0);
+  grid_draw_cell_pointer (grid, col, row, mask);
+
+  grid->pointed_col = col;
+  grid->pointed_row = row;
+  grid->mask_pointer = mask;
+}
+
 static gboolean
 grid_button_press_event (GtkWidget *widget, GdkEventButton *event, grid_t *grid)
 {
@@ -757,29 +804,8 @@ grid_button_press_event (GtkWidget *widget, GdkEventButton *event, grid_t *grid)
     int row = grid_y2row (grid, event->y);
     if (col >= 0 && col < grid->col_num && row >= 0 && row < grid->row_num)
      {
-      if (event->state & GDK_SHIFT_MASK) 
-       {
-        grid->last_mask = grid->cells[row][col].mask = grid->cells[row][col].mask ? 0 : 1;
-        if (grid->mask_changed_cb)
-          grid->mask_changed_cb (grid->mask_changed_data, col, row, grid->cells[row][col].mask);
-       }
-      else
-       {
-        grid->last_value = grid->cells[row][col].value = grid->cells[row][col].value ? 0 : 1;
-        if (grid->value_changed_cb)
-          grid->value_changed_cb (grid->value_changed_data, col, row, grid->cells[row][col].value);
-       }
-
-      grid_draw_cell (grid, col, row, 1, 0);
-      grid->last_col = col;
-      grid->last_row = row;
-      grid_draw_cell_pointer (grid, col, row, event->state & GDK_SHIFT_MASK);
-      if (grid->pointed_col != -1 && grid->pointed_row != -1 &&
-          (grid->pointed_col != col || grid->pointed_row != row))
-        grid_draw_cell (grid, grid->pointed_col, grid->pointed_row, 1, 0);
-
-      grid->pointed_col = col;
-      grid->pointed_row = row;
+      grid_toggle_cell (grid, col, row, event->state & GDK_SHIFT_MASK);
+      grid_move_pointer (grid, col, row, event->state & GDK_SHIFT_MASK);
      }
    }
   
@@ -797,65 +823,85 @@ grid_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, grid_t *grid
   if (event->is_hint)
     gdk_window_get_pointer (event->window, &x, &y, &state);
   else
-    {
-      x = event->x;
-      y = event->y;
-      state = event->state;
-    }
+   {
+    x = event->x;
+    y = event->y;
+    state = event->state;
+   }
     
-  int col = grid_x2col (grid, event->x);
-  int row = grid_y2row (grid, event->y);
+  int col = grid_x2col (grid, x);
+  int row = grid_y2row (grid, y);
 
   if (col >= 0 && col < grid->col_num && row >= 0 && row < grid->row_num)
    {
     if (state & GDK_BUTTON1_MASK && ((col != grid->last_col) || (row != grid->last_row)))
      {
-      //grid_handle_pointer_action (grid, event->x, event->y, 0);
-      if (state & GDK_SHIFT_MASK)  
-       {
-        if ((grid->cells[row][col].mask != grid->last_mask))
-         {
-          grid->last_mask = grid->cells[row][col].mask = grid->cells[row][col].mask ? 0 : 1;
-          grid_draw_cell (grid, col, row, 1, 0);
-          grid->last_col = col;
-          grid->last_row = row;
-          if (grid->mask_changed_cb)
-            grid->mask_changed_cb (grid->mask_changed_data, col, row, grid->cells[row][col].mask);
-         }
-       }
+      if ((state & GDK_SHIFT_MASK) && (grid->cells[row][col].mask != grid->last_mask))  
+        grid_toggle_cell (grid, col, row, 1);
       else if (grid->cells[row][col].value != grid->last_value)
-       {
-        grid->last_value = grid->cells[row][col].value = grid->cells[row][col].value ? 0 : 1;
-        grid_draw_cell (grid, col, row, 1, 0);
-        grid->last_col = col;
-        grid->last_row = row;
-        if (grid->value_changed_cb)
-          grid->value_changed_cb (grid->value_changed_data, col, row, grid->cells[row][col].value);
-       }
+        grid_toggle_cell (grid, col, row, 0);
      } 
-//    else if (col != grid->pointed_col || row != grid->pointed_row)
-//     {
-      grid_draw_cell_pointer (grid, col, row, state & GDK_SHIFT_MASK);
-      if (grid->pointed_col != -1 && grid->pointed_row != -1 &&
-          (grid->pointed_col != col || grid->pointed_row != row))
-        grid_draw_cell (grid, grid->pointed_col, grid->pointed_row, 1, 0);
 
-      grid->pointed_col = col;
-      grid->pointed_row = row;
-//     }
+    grid_move_pointer (grid, col, row, event->state & GDK_SHIFT_MASK);
+   }
+ 
+  gtk_widget_grab_focus (grid->area);
+
+  return TRUE;
+}
+
+static gboolean
+grid_key_action_event (GtkWidget * widget, GdkEventKey *event, grid_t * grid)
+{
+  int col = grid->pointed_col;
+  int row = grid->pointed_row;
+  int mask = event->state & GDK_SHIFT_MASK;
+  int toggle = 0;
+  if (event->type == GDK_KEY_PRESS) 
+   {
+    switch (event->keyval) 
+     {
+      case GDK_Up:    row--; break;
+      case GDK_Right: col++; break;
+      case GDK_Down:  row++; break;
+      case GDK_Left:  col--; break;
+      case GDK_Shift_L:  
+      case GDK_Shift_R:  
+        mask = 1;
+        break;
+      case GDK_Return:
+        toggle = 1;
+        break;
+      //FIXME: what about GDK_Delete as an eraser?
+      default:
+        return FALSE;
+     }
    }
   else
    {
-    /*
-    if (grid->pointed_col != -1 && grid->pointed_row != -1)
+    switch (event->keyval) 
      {
-      grid_draw_cell (grid, grid->pointed_col, grid->pointed_row, 1);
-      grid->pointed_col = -1;
-      grid->pointed_row = -1;
+      case GDK_Shift_L:  
+      case GDK_Shift_R:  
+        mask = 0;
+        break;
+      default:
+        return FALSE;
      }
-     */
    }
-  
+  if (toggle) 
+   {
+    if (col >= 0 && col < grid->col_num && row >= 0 && row < grid->row_num)
+      grid_toggle_cell (grid, col, row, mask);
+   }
+  else
+   {
+    row = row < 0 ? 0 : (row >= grid->row_num ? grid->row_num - 1  : row);
+    col = col < 0 ? 0 : (col >= grid->col_num ? grid->col_num - 1  : col);
+   }
+
+  grid_move_pointer (grid, col, row, mask);
+
   return TRUE;
 }
 
