@@ -1,7 +1,7 @@
 /*
  *   Jackbeat - JACK sequencer
  *    
- *   Copyright (c) 2004-2006 Olivier Guilyardi <olivier {at} samalyse {dot} com>
+ *   Copyright (c) 2004-2008 Olivier Guilyardi <olivier {at} samalyse {dot} com>
  *    
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   SVN:$Id: rc.c 68 2007-03-01 12:02:41Z olivier $
+ *   SVN:$Id: rc.c 124 2008-01-10 19:10:45Z olivier $
  */
 
 #include <stdio.h>
@@ -76,24 +76,25 @@ rc_read (rc_t *rc)
     {
       while (fgets (s, 256, fd))
         {
-          sscanf (s, "%s = %s", key, val);
-          
-          if (strcmp (key, "sample_wdir") == 0) 
-            strcpy (rc->sample_wdir, val);
-          else if (strcmp (key, "pattern_wdir") == 0) 
-            strcpy (rc->sequence_wdir, val);
-          else if ((strcmp (key, "sample_history") == 0) && rc->sample_history_num < 10) 
-            strcpy (rc->sample_history[rc->sample_history_num++], val);
-          else if ((strcmp (key, "sequence_history") == 0) && rc->sequence_history_num < 10) 
-            strcpy (rc->sequence_history[rc->sequence_history_num++], val);
-          else if (strcmp (key, "default_resampler_type") == 0)
-            rc->default_resampler_type = util_str_to_resampler_type (val);
-          else if (strcmp (key, "transport_aware") == 0)
-            sscanf (val, "%d", &(rc->transport_aware));
-          else if (strcmp (key, "transport_query") == 0)
-            sscanf (val, "%d", &(rc->transport_query));
-          else if (strcmp (key, "auto_connect") == 0)
-            sscanf (val, "%d", &(rc->auto_connect));
+          if ((sscanf (s, "%s = \"%[^\"]\"", key, val) == 2) || (sscanf (s, "%s = %s", key, val) == 2))
+           {
+            if (strcmp (key, "sample_wdir") == 0) 
+              strcpy (rc->sample_wdir, val);
+            else if (strcmp (key, "pattern_wdir") == 0) 
+              strcpy (rc->sequence_wdir, val);
+            else if ((strcmp (key, "sample_history") == 0) && rc->sample_history_num < 10) 
+              strcpy (rc->sample_history[rc->sample_history_num++], val);
+            else if ((strcmp (key, "sequence_history") == 0) && rc->sequence_history_num < 10) 
+              strcpy (rc->sequence_history[rc->sequence_history_num++], val);
+            else if (strcmp (key, "default_resampler_type") == 0)
+              rc->default_resampler_type = util_str_to_resampler_type (val);
+            else if (strcmp (key, "transport_aware") == 0)
+              sscanf (val, "%d", &(rc->transport_aware));
+            else if (strcmp (key, "transport_query") == 0)
+              sscanf (val, "%d", &(rc->transport_query));
+            else if (strcmp (key, "auto_connect") == 0)
+              sscanf (val, "%d", &(rc->auto_connect));
+           }
         }
       fclose (fd);
       if (stat (path, &b) == 0 && S_ISREG (b.st_mode)) {
@@ -123,12 +124,12 @@ rc_write (rc_t * rc)
   sprintf (s, "%s/.jackbeat/config", getenv ("HOME"));
   if ((fd = fopen (s, "w"))) 
    {
-    if (strlen (rc->sample_wdir)) fprintf (fd, "sample_wdir = %s\n", rc->sample_wdir);
-    if (strlen (rc->sequence_wdir)) fprintf (fd, "pattern_wdir = %s\n", rc->sequence_wdir);
+    if (strlen (rc->sample_wdir)) fprintf (fd, "sample_wdir = \"%s\"\n", rc->sample_wdir);
+    if (strlen (rc->sequence_wdir)) fprintf (fd, "pattern_wdir = \"%s\"\n", rc->sequence_wdir);
     for (i=0; i < rc->sample_history_num; i++)
-      fprintf (fd , "sample_history = %s\n", rc->sample_history[i]);
+      fprintf (fd , "sample_history = \"%s\"\n", rc->sample_history[i]);
     for (i=0; i < rc->sequence_history_num; i++)
-      fprintf (fd , "sequence_history = %s\n", rc->sequence_history[i]);
+      fprintf (fd , "sequence_history = \"%s\"\n", rc->sequence_history[i]);
     if (rc->default_resampler_type != -1)
       fprintf (fd, "default_resampler_type = %s\n", 
                util_resampler_type_to_str (rc->default_resampler_type));
