@@ -1,7 +1,7 @@
 /*
  *   Jackbeat - JACK sequencer
  *    
- *   Copyright (c) 2004-2006 Olivier Guilyardi <olivier {at} samalyse {dot} com>
+ *   Copyright (c) 2004-2008 Olivier Guilyardi <olivier {at} samalyse {dot} com>
  *    
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   SVN:$Id: sequence.h 50 2006-12-03 23:44:12Z olivier $
+ *   SVN:$Id: sequence.h 121 2008-01-10 00:17:40Z olivier $
  */
 
 #ifndef JACKBEAT_SEQUENCE_H
@@ -27,9 +27,13 @@
 #include "sample.h"
 #include "types.h"
 
-#define SEQUENCE_SINC 1
+#define SEQUENCE_SINC   1
 #define SEQUENCE_LINEAR 2
 
+#define SEQUENCE_SUSTAIN_LOOP     1
+#define SEQUENCE_SUSTAIN_TRUNCATE 2
+#define SEQUENCE_SUSTAIN_KEEP     3
+
 typedef struct sequence_t sequence_t;
 
 typedef enum sequence_track_type_t 
@@ -44,7 +48,9 @@ sequence_t *  sequence_new (char *name, int *error);
 void          sequence_destroy (sequence_t *sequence);
 
 /* Waveform export */
-void          sequence_export (sequence_t *sequence, char *filename, progress_callback_t progress_callback, void *progress_data);
+void          sequence_export (sequence_t *sequence, char *filename, int framerate,
+                               int sustain_type,
+                               progress_callback_t progress_callback, void *progress_data);
 
 /* Global settings and informations */
 char *          sequence_get_name (sequence_t *sequence);
@@ -53,6 +59,7 @@ char            sequence_is_auto_connecting (sequence_t *sequence);
 jack_nframes_t  sequence_get_framerate (sequence_t *sequence);
 void            sequence_set_resampler_type (sequence_t * sequence, int type);
 int             sequence_get_resampler_type (sequence_t *sequence);
+int             sequence_get_error (sequence_t * sequence);
 
 /* Transport control */
 void    sequence_set_transport (sequence_t *sequence, char respond, char query);
@@ -62,7 +69,7 @@ void    sequence_rewind (sequence_t *sequence);
 int     sequence_is_playing (sequence_t *sequence);
 
 /* Dimensions, BPM and looping */
-void    sequence_resize (sequence_t *sequence, int tracks_num, int beats_num, 
+int     sequence_resize (sequence_t *sequence, int tracks_num, int beats_num, 
                          int measure_len, int duplicate_beats);
 void    sequence_remove_track (sequence_t *sequence, int track);
 int     sequence_get_tracks_num (sequence_t *sequence);
@@ -83,7 +90,7 @@ sequence_track_type_t sequence_get_track_type (sequence_t *sequence, int track);
 
 /* Samples */
 sample_t *  sequence_get_sample (sequence_t *sequence, int track);
-void        sequence_set_sample (sequence_t *sequence, int track, sample_t *sample);
+int         sequence_set_sample (sequence_t *sequence, int track, sample_t *sample);
 int         sequence_get_sample_usage (sequence_t *sequence, sample_t *sample);
 
 /* Track volume, pitch and smoothing */
